##!/bin/bash
## CPU Frequency scaling script
## Requires bc for status calculations
## $driver must be set for this to work

statpath=/sys/devices/system/cpu/cpu0/cpufreq
#driverfixme

stats() {
	curfreq="$(cat ${statpath}/cpuinfo_cur_freq)"
	maxfreq="$(cat ${statpath}/cpuinfo_max_freq)"
	minfreq="$(cat ${statpath}/cpuinfo_min_freq)"
	maxscal="$(cat ${statpath}/scaling_max_freq)"
	minscal="$(cat ${statpath}/scaling_min_freq)"
	avafreq=($(cat ${statpath}/scaling_available_frequencies))
	driscal="$(cat ${statpath}/scaling_driver)"
	setscal="${statpath}/scaling_setspeed"
	pid="$(pidof powerthend)"
	
	echo ""
	echo -n "Daemon Running:"
	if [ -n "${pid}" ]; then
		echo -e "\t\t\bYes\n"
	else
		echo -e "\t\t\bNo\n"
	fi
	
	echo -n "Current Frequency:"
	echo -e "\t\b${curfreq:0:1}.${curfreq:1:2}ghz\n"
	
	echo -n "CPU Max:"
	echo -e "\t\t\b${maxfreq:0:1}.${maxfreq:1:2}ghz\n"
	
	echo -n "Scaling Max:"
	echo -e "\t\t\b${maxscal:0:1}.${maxscal:1:2}ghz\n"
	
	echo -n "CPU Min:"
	echo -e "\t\t\b${minfreq:0:1}.${minfreq:1:2}ghz\n"
	
	echo -n "Scaling Min:"
	echo -e "\t\t\b${minscal:0:1}.${minscal:1:2}ghz\n"
	
	echo -n "Avaliable Frequencies:"
	
	i=0
	unset damnhack
	while [ -n "${avafreq[$i]}" ]; do
		echo -e " ${damnhack}${avafreq[$i]:0:1}.${avafreq[$i]:1:2}ghz"
		damnhack="\t\t\t\b"
		let i++;
	done
	
	echo ""
	echo -n "Current Driver:"
	echo -e "\t\t\b${driscal}\n"
}

modcheck() {
	if [ -z "$( lsmod | grep ${driver} )" ]; then
		modprobe ${driver}
	fi
}

setspeed() {
	echo "${1}" > ${statpath}/scaling_setspeed
}

case ${1} in
	start|restart|dyn|low|high)
		killall -15 powerthend >& /dev/null
		modcheck
		case ${1} in
			start|restart|dyn)
				/usr/sbin/powerthend
				echo $(cat ${statpath}/cpuinfo_min_freq) > ${statpath}/scaling_min_freq
				echo $(cat ${statpath}/cpuinfo_max_freq) > ${statpath}/scaling_max_freq
				;;
			low)
				setspeed $(cat ${statpath}/cpuinfo_min_freq)
				;;
			high)
				setspeed $(cat ${statpath}/cpuinfo_max_freq)
				;;
		esac
		;;
	stop)
		killall -15 powerthend >& /dev/null
		setspeed $(cat ${statpath}/cpuinfo_max_freq)
		;;
	status)
		modcheck
		stats
		;;
	*)
		echo "Usage: (start|stop|dyn|low|high|restart|status)"
		;;
esac
